# Scrollbar.w -- scrollbar widget
# Original Author: Bert Bos <bert@let.rug.nl>
# Significantly rewritten by Joel N. Weber II <nemo@koa.iolani.honolulu.hi.us>
# Copyright: see README file

@CLASS XfwfScrollbar (XfwfBoard)  @file = Scrollbar

@ The scrollbar widget helps the user to view data that is too large
to be displayed all at once.  They are often used through a
ScrolledWindow (see there), where they are put next to and/or below
the widget that contains the data. The scrollbar controls which part
is visible. By manipulating the scrollbar the user can move (`scroll')
the data within the other widget.

The |XfwfScrollbar| widget simply creates several children which
all understand SWIP.  Most often, a |XfwfSlider2| widget is used
along with two |XfwfSWIPButton| widgets, but this design allows
the user much freedom in the exact arrangement.


@PUBLIC

@ This callback will be called whenever any child of the |XfwfScrollbar|
is called.

	@var <Callback> XtCallbackList scrollCallback = NULL

@ The standard way to update the scrollbar is with a call to the
function that is knwon as the |scrollResponse| function. To get a
pointer to that function, you should use |XtGetValues| or
|XtVaGetValues| on the |XtNscrollResponse| resource. But from an
application it might be easier to use the |XfwfSetScrollbar|
convenience function instead.

The |scrollResponse| routine has the same format as a callback
procedure. The first argument is a widget, this argument is ignored.
The second argument is |XtPointer client_data|, it must point to the
scrollbar that is to be updated. The third argument is |XtPointer
call_data|, it must be a pointer to an |XfwfScrollInfo| structure (see
above).

	@var XtCallbackProc scrollResponse = scroll_response

@ The |childList| resource is used to list the children to be created.
Here is an example of the format:

|thumb: Slider2; up: Button; down: Button|

	@var String childList = ""

@PRIVATE

@ The |scrollResponse| function of the Slider2 that implements the
thumb is stored in a private variable.

	@var XtCallbackProc child_scroll

@ The |responseCallback| is called by |scrollResponse| to distribute the
message to all the children.

	@var XtCallbackList responseCallback

@EXPORTS

# FIXME: XfwfSetScrollbar needs to be rewritten
@ The |XfwfSetScrollbar| convenience function can be used to set the
position and size of a scrollbar. The two arguments must be between
0.0 and 1.0 (inclusive).

@proc XfwfSetScrollbar($, double pos, double size)
{
}

@UTILITIES

@ The |scroll_response| method is accessed via the |scrollResponse|
resource. It is passed the scrollbar itself as the |client_data|
argument and a pointer to an |XfwfScrollInfo| record as |call_data|.

The scrollbar passes on the call to the Slider2 widget that is
managing the thumb.

@proc scroll_response($, XtPointer client_data, XtPointer call_data)
{
	XtCallCallbackList(client_data, $scrollResponse, call_data);
}

@ The |thumbscroll| routine is the callback for the scroll callback of
the slider. It invokes the scrollbar's |scrollCallback|, after making
sure that only appropriate position information is passed. (The
Slider2 might pass horizontal position where only vertical position is
relevant, and vice versa.)

@proc thumbscroll($, XtPointer client_data, XtPointer call_data)
{
    XtCallCallbackList(client_data, $scrollCallback, call_data);
}


@METHODS

@proc initialize
{
	char *name, *type, *s, *q;
	int i;
	void *class;
	Widget child;

	/* Make sure we have enough space in tmp buffers */
	s = $childList;
	i = strlen(s) + 1;
	name = malloc(i);
	type = malloc(i);

	while (*s) {
		/* Extract the name */
		while ((*s == ' ') || (*s == '\t') || (*s == '\n')) s++;
		q = s;
		while (*q && (*q != ' ') && (*q != '\t') && (*q != '\n')) q++;
		strncpy(name, s, q - s);
		name[q - s] = 0;
		s = q;
		while ((*s == ' ') || (*s == '\t') || (*s == '\n')) s++;
		if (*s == ':') {
			s++;
		} else {
			fprintf(stderr,
				"XfwfScrollbar: misformatted childList\n");
		}
		while ((*s == ' ') || (*s == '\t') || (*s == '\n')) s++;
		
		q = s;
		while (*q && (*q != ' ') && (*q != '\t') && (*q != '\n')) q++;
		strncpy(type, s, q - s);
		type[q - s] = 0;
		q = s;
		while ((*s == ' ') || (*s == '\t') || (*s == '\n')) s++;
		if (*s == ';') {
			s++;
		} else if (*s != ';') {
			fprintf(stderr,
				"XfwfScrollbar: misformatted childList");
		}

		if (!strcmp(type, "button")) {
			class = xfwfSWIPButtonWidgetClass;
		} else if (!strcmp(type, "slider2")) {
			class = xfwfSlider2WidgetClass;
		} else if (!strcmp(type, "slider4")) {
			class = xfwfSlider4WidgetClass;
		/* FIXME more types needed */
		} else {
			fprintf(stderr, "Unknown widget type: %s\n", type);
			class = xfwfSlider2WidgetClass;
		}

		child = XtVaCreateManagedWidget(name, class, $, NULL, 0);
		XtAddCallback(child, XtNscrollCallback, scroll_response, $);
	}
}

# FIXME add set_values



@IMPORTS

@incl <Xfwf/Slider2.h>
@incl <Xfwf/Slider4.h>
@incl <Xfwf/SWIPBtn.h>
@incl <stdio.h>
@incl <ctype.h>
